$: << File.dirname(File.expand_path(__FILE__))

require "ev/oldandnewlocation"
require "ev/dependencies"
require "ev/ftools"
require "rbconfig"

exit	if ARGV.include?("--exit")

def backslashes(s)
  s	= s.gsub(/^\.\//, "").gsub(/\//, "\\\\")	if windows?
  s
end

def linux?
  not darwin? and not windows? and not cygwin?	# Hack ???
end

def darwin?
  not (target_os.downcase =~ /darwin/).nil?	# Hack ???
end

def windows?
  not (target_os.downcase =~ /32/).nil?		# Hack ???
end

def cygwin?
  not (target_os.downcase =~ /cyg/).nil?	# Hack ???
end

def target_os
  Config::CONFIG["target_os"] or ""
end

rubyw	= false
rubyw	= false		if (ARGV.include?("-d") or ARGV.include?("--ruby"))
rubyw	= true		if (ARGV.include?("-w") or ARGV.include?("--rubyw"))
site	= false
site	= true		if (ARGV.include?("-s") or ARGV.include?("--site"))

ARGV.delete_if{|s| s	=~ /^-/}

exefile		= (ARGV.shift or (windows? ? "allinoneruby.exe" : "allinoneruby.bin"))

bindir1		= Config::CONFIG["bindir"]
libdir1		= Config::CONFIG["libdir"]
bindir2		= tmplocation("bin/")
libdir2		= tmplocation("lib/")
rubylibdir1	= Config::CONFIG["rubylibdir"]
sitelibdir1	= Config::CONFIG["sitelibdir"]

Dir.mkdir(bindir2)	unless File.directory?(bindir2)
Dir.mkdir(libdir2)	unless File.directory?(libdir2)

if linux?
  rubyexe	= "#{bindir1}/ruby"
else
  rubyexe	= "#{bindir1}/ruby.exe"
  rubywexe	= "#{bindir1}/rubyw.exe"
end

if linux?
  tocopy	= ldds(rubyexe)
  tocopy << rubyexe	if File.file?(rubyexe)
else
  tocopy	= dlls(rubyexe)
  tocopy << rubyexe	if File.file?(rubyexe)
  tocopy << rubywexe	if File.file?(rubywexe)
end

tocopy.each do |s1|
  file	= File.basename(s1)
  s2	= File.expand_path(file, bindir2)

  #$stderr.puts "Copying #{s1} ..."
  File.copy(s1, s2)	unless File.file?(s2)
end

begin
  file	= rubylibdir1[libdir1.length..-1].gsub(/^[\/\\]*/, "")
  s1	= rubylibdir1
  s2	= File.expand_path(file, libdir2)

  #$stderr.puts "Copying #{s1}/ ..."
  Dir.copy(s1, s2)	unless File.directory?(s2)
end

if site
  file	= sitelibdir1[libdir1.length..-1].gsub(/^[\/\\]*/, "")
  s1	= sitelibdir1
  s2	= File.expand_path(file, libdir2)

  #$stderr.puts "Copying #{s1}/ ..."
  Dir.copy(s1, s2)	unless File.directory?(s2)
end

if linux?
  tocopy	= Dir.find(libdir2, /\.(so|o|dll)$/i).collect{|file| ldds(file)}.flatten.sort.uniq
else
  tocopy	= Dir.find(libdir2, /\.(so|o|dll)$/i).collect{|file| dlls(file, bindir1)}.flatten.sort.uniq
end

tocopy.each do |s1|
  file	= File.basename(s1)
  s2	= File.expand_path(file, bindir2)

  #$stderr.puts "Copying #{s1} ..."
  File.copy(s1, s2)	unless File.file?(s2)
end

eeeexe	= "eee.exe"
eeeexe	= "eeew.exe"	if rubyw
eeeexe	= "eee.bin"	if linux?
appexe	= exefile

$stderr.puts "Creating #{appexe} ..."

if linux?
  File.open(tmplocation("eee.sh"), "w") do |f|
    f.puts "PDIR=$1;shift"
    f.puts "DIR=$(pwd)"
    f.puts "cd $PDIR"
    f.puts "  chmod +x bin/ruby"
    f.puts "  export PATH=$(pwd)/bin:$PATH"
    f.puts "  export LD_LIBRARY_PATH=$(pwd)/bin:LD_LIBRARY_PATH"
    f.puts "cd $DIR"
    f.puts "$*"
  end
end

File.open(tmplocation("eee.rb"), "w") do |f|
  f.puts "lib	= File.expand_path(File.dirname(__FILE__)) + '/lib'"
  f.puts "lib.sub!(/^.:/, '/cygdrive/%s' % $&[0..0].downcase)	if lib =~ /^.:/"	if cygwin?
  f.puts "$:.clear"
  $:.each do |s|
    f.puts "$: << \"%s\"" % s.sub(libdir1, '#{lib}')
  end
end

File.open(tmplocation("bootstrap.rb"), "w") do |f|
  f.puts "load($0 = ARGV.shift)"
end

File.open(tmplocation("empty.rb"), "w") do |f|
end

File.open(tmplocation("app.eee"), "w") do |f|
  rubyexe	= "ruby.exe"
  rubyexe	= "rubyw.exe"	if rubyw
  rubyexe	= "ruby"	if linux?

  f.puts "r bin"
  f.puts "r lib"
  f.puts "f eee.sh"	if linux?
  f.puts "f eee.rb"
  f.puts "f bootstrap.rb"
  f.puts "f empty.rb"
  if linux?
    f.puts "c source %tempdir%/eee.sh %tempdir% %tempdir%/bin/#{rubyexe} -r %tempdir%/eee.rb -r %tempdir1%/bootstrap.rb -T1 %tempdir1%/empty.rb %quotedparms%"
  elsif cygwin?
    f.puts "c %tempdir%\\bin\\#{rubyexe} -r %tempdir1%/eee.rb -r %tempdir1%/bootstrap.rb -T1 %tempdir1%/empty.rb %quotedparms%"
  else
    f.puts "c %tempdir%\\bin\\#{rubyexe} -r %tempdir%\\eee.rb -r %tempdir%\\bootstrap.rb -T1 %tempdir%\\empty.rb %quotedparms%"
  end
end

from	= newlocation(eeeexe)
to	= tmplocation(eeeexe)

File.copy(from, to)	unless from == to
File.chmod(0755, to)	if linux?

tmplocation do
  ENV["EEE_EXE"]	= eeeexe
  ENV["EEE_DIR"]	= Dir.pwd

  eeebin1	= newlocation("eee.exe")
  eeebin1	= newlocation("eee.bin")	if linux?
  eeebin1	= newlocation("eee.bin2")	if darwin?

  eeebin2	= tmplocation("eee.exe")
  eeebin2	= tmplocation("eee.bin")	if linux?
  eeebin2	= tmplocation("eee.bin")	if darwin?

  from	= eeebin1
  to	= eeebin2

  File.copy(from, to)	unless from == to
  File.chmod(0755, to)	if linux? or darwin?

  system(backslashes("#{eeebin2} app.eee #{appexe}"))
end

from	= tmplocation(appexe)
to	= oldlocation(appexe)

File.copy(from, to)	unless from == to
File.chmod(0755, to)	if linux?
