$: << File.dirname(File.expand_path(__FILE__))

require "ev/oldandnewlocation"
require "ev/dependencies"
require "ev/ftools"
require "rbconfig"
require "rubyscript2exe"

exit	if RUBYSCRIPT2EXE.is_compiling?

def backslashes(s)
  s	= s.gsub(/^\.\//, "").gsub(/\//, "\\\\")	if windows?
  s
end

def linux?
  not darwin? and not windows? and not cygwin?
end

def darwin?
  not (target_os.downcase =~ /darwin/).nil?
end

def windows?
  not (target_os.downcase =~ /32/).nil?
end

def cygwin?
  not (target_os.downcase =~ /cyg/).nil?
end

def target_os
  Config::CONFIG["target_os"] or ""
end

def copyto(files, dest)
  [files].flatten.sort.uniq.each do |fromfile|
    tofile	= File.expand_path(File.basename(fromfile), dest)

    $stderr.puts "Copying #{fromfile} ..."	if VERBOSE

    File.copy(fromfile, tofile)			unless File.file?(tofile)
  end
end

rubyw	= false
rubyw	= false		if (ARGV.include?("-d") or ARGV.include?("--ruby"))
rubyw	= true		if (ARGV.include?("-w") or ARGV.include?("--rubyw"))
site	= false
site	= true		if (ARGV.include?("-s") or ARGV.include?("--site"))

STRIP	= (not (ARGV.include?("-ns") or ARGV.include?("--nostrip")))
VERBOSE	= (ARGV.include?("-v") or ARGV.include?("--verbose"))
QUIET	= ((ARGV.include?("-q") or ARGV.include?("--quiet")) and not VERBOSE)

ARGV.delete_if{|s| s =~ /^-/}

exefile		= ARGV.shift
exefile		= (exefile or "allinoneruby.exe")	if windows? or cygwin?
exefile		= (exefile or "allinoneruby_linux")	if linux?
exefile		= (exefile or "allinoneruby_darwin")	if darwin?

bindir1		= Config::CONFIG["bindir"]
libdir1		= Config::CONFIG["libdir"]
sitedir1	= Config::CONFIG["sitedir"]
bindir2		= tmplocation("bin/")
libdir2		= tmplocation("lib/")

numberoflibs	= 0

Dir.mkdir(bindir2)	unless File.directory?(bindir2)
Dir.mkdir(libdir2)	unless File.directory?(libdir2)

if linux? or darwin?
  rubyexe	= "#{bindir1}/ruby"
else
  rubyexe	= "#{bindir1}/ruby.exe"
  rubywexe	= "#{bindir1}/rubyw.exe"
end

$stderr.puts "Copying files..."	unless QUIET

begin
  $:.each do |dir|
    if (dir.include?(libdir1) and not dir.include?(sitedir1)) or (site and dir.include?(sitedir1))
      numberoflibs += 1
      s1	= dir
      s2	= File.expand_path("lib#{numberoflibs}", libdir2)

      $stderr.puts "Copying #{s1}/ ..."	if VERBOSE

      Dir.copy(s1, s2)	unless File.directory?(s2)
    end
  end
end

copyto(rubyexe, bindir2)	if linux? or darwin?	if File.file?(rubyexe)
copyto(ldds(rubyexe), bindir2)	if linux? or darwin?

copyto(rubyexe, bindir2)	if windows? or cygwin?	if File.file?(rubyexe)
copyto(rubywexe, bindir2)	if windows? or cygwin?	if File.file?(rubyexe)
copyto(dlls(rubyexe), bindir2)	if windows? or cygwin?

copyto(Dir.find(libdir2, /\.(so|o|dll)$/i).collect{|file| ldds(file)}, bindir2)	if linux? or darwin?
copyto(Dir.find(libdir2, /\.(so|o|dll)$/i).collect{|file| dlls(file)}, bindir2)	if windows? or cygwin?

if STRIP and (linux? or darwin?)
  $stderr.puts "Stripping..."	unless QUIET

  system("cd #{bindir2} ; strip --strip-all * 2> /dev/null")
  system("cd #{libdir2} ; strip --strip-all * 2> /dev/null")
end

rubyexe	= "ruby.exe"
rubyexe	= "rubyw.exe"		if rubyw
rubyexe	= "ruby"		if linux?
rubyexe	= "ruby"		if darwin?
eeeexe	= "eee.exe"
eeeexe	= "eeew.exe"		if rubyw
eeeexe	= "eee_linux"		if linux?
eeeexe	= "eee_darwin"		if darwin?
appexe	= exefile

$stderr.puts "Creating #{appexe} ..."	unless QUIET

File.open(tmplocation("bootstrap.rb"), "w") do |f|
  f.puts "# Set up the environment"

  f.puts "module ALLINONERUBY"
  f.puts "end"

  f.puts "lib	= File.expand_path(File.dirname(__FILE__)) + '/lib'"
  f.puts "lib.sub!(/^.:/, '/cygdrive/%s' % $&[0..0].downcase)	if lib =~ /^.:/"	if cygwin?
  f.puts "$:.clear"

  numberoflibs.times do |n|
    f.puts "$: << lib + '/lib%s'" % [n+1]
  end
  f.puts "$: << '.'"

  f.puts "# Load eee.info"

  f.puts "eeedir		= File.dirname(__FILE__)"
  f.puts "eeeinfo		= File.expand_path('eee.info', eeedir)"
  f.puts "if File.file?(eeeinfo)"
  f.puts "  lines	= File.open(eeeinfo){|f| f.readlines}"
  f.puts "  badline	= lines.find{|line| line !~ /^EEE_/}"
  f.puts "  while badline"
  f.puts "    pos		= lines.index(badline)"
  f.puts "    raise 'Found badline at position 0.'	if pos == 0"
  f.puts "    lines[pos-1..pos]	= lines[pos-1] + lines[pos]"
  f.puts "    badline		= lines.find{|line| line !~ /^EEE_/}"
  f.puts "  end"
  f.puts "  lines.each do |line|"
  f.puts "    k, v	= line.strip.split(/\s*=\s*/, 2)"
  f.puts "    k.gsub!(/^EEE_/, '')"
  f.puts "    ALLINONERUBY.module_eval{const_set(k, v)}"
  f.puts "  end"
  f.puts "  ARGV.concat(ALLINONERUBY::PARMSLIST.split(/\000/))"
  f.puts "end"

  f.puts "# Fake Config"

  #f.puts "module Config"
  #f.puts "  DESTDIR = ALLINONERUBY::TEMPDIR"
  #f.puts "end"

  f.puts "require 'rbconfig'"
  f.puts "Config::CONFIG['bindir'] = File.join(ALLINONERUBY::TEMPDIR, 'bin')"
  f.puts "Config::CONFIG['libdir'] = File.join(ALLINONERUBY::TEMPDIR, 'lib')"

  f.puts "module ALLINONERUBY"
  f.puts "  RUBYEXE	= '#{rubyexe}'"
  f.puts "end"

  f.puts "# Start the application"

  f.puts "script	= ARGV.shift"
  f.puts "if script == '-e'"
  f.puts "  command	= ARGV.shift"
  f.puts "  eval(command)"
  f.puts "else"
  f.puts "  if script.nil? or script == '-'"
  f.puts "    command	= $stdin.read"
  f.puts "    eval(command)"
  f.puts "  else"
  f.puts "    script	= File.expand_path(script)"
  f.puts "    $0.replace(script)"
  f.puts "    load(script)"
  f.puts "  end"
  f.puts "end"
end

File.open(tmplocation("empty.rb"), "w") do |f|
end

File.open(tmplocation("app.eee"), "w") do |f|
  f.puts "r bin"
  f.puts "r lib"
  f.puts "f bootstrap.rb"
  f.puts "f empty.rb"
  f.puts "i eee.info"

  if linux?
    f.puts "c PATH=%tempdir%/bin:$PATH ; export LD_LIBRARY_PATH=%tempdir%/bin:$LD_LIBRARY_PATH ; chmod +x %tempdir%/bin/* ; %tempdir%/bin/#{rubyexe} -r %tempdir%/bootstrap.rb -T1 %tempdir%/empty.rb"
  elsif darwin?
    f.puts "c PATH=%tempdir%/bin:$PATH ; export DYLD_LIBRARY_PATH=%tempdir%/bin:$DYLD_LIBRARY_PATH ; chmod +x %tempdir%/bin/* ; %tempdir%/bin/#{rubyexe} -r %tempdir%/bootstrap.rb -T1 %tempdir%/empty.rb"
  elsif cygwin?
    f.puts "c %tempdir%\\bin\\#{rubyexe} -r %tempdir1%/bootstrap.rb -T1 %tempdir1%/empty.rb"
  else
    f.puts "c %tempdir%\\bin\\#{rubyexe} -r %tempdir%\\bootstrap.rb -T1 %tempdir%\\empty.rb"
  end
end

from	= newlocation(eeeexe)
from	= applocation(eeeexe)	unless File.file?(from)
from	= oldlocation(eeeexe)	unless File.file?(from)
to	= tmplocation(eeeexe)

File.copy(from, to)	unless from == to
File.chmod(0755, to)	if linux? or darwin?

tmplocation do
  ENV["EEE_EXE"]	= eeeexe
  ENV["EEE_DIR"]	= Dir.pwd

  eeebin1	= newlocation("eee.exe")
  eeebin1	= newlocation("eee_linux")	if linux?
  eeebin1	= newlocation("eee_darwin")	if darwin?

  unless File.file?(eeebin1)
    eeebin1	= applocation("eee.exe")
    eeebin1	= applocation("eee_linux")	if linux?
    eeebin1	= applocation("eee_darwin")	if darwin?
  end

  unless File.file?(eeebin1)
    eeebin1	= oldlocation("eee.exe")
    eeebin1	= oldlocation("eee_linux")	if linux?
    eeebin1	= oldlocation("eee_darwin")	if darwin?
  end

  eeebin2	= tmplocation("eee.exe")
  eeebin2	= tmplocation("eee_linux")	if linux?
  eeebin2	= tmplocation("eee_darwin")	if darwin?

  from	= eeebin1
  to	= eeebin2

  File.copy(from, to)	unless from == to
  File.chmod(0755, to)	if linux? or darwin?

  system(backslashes("#{eeebin2} app.eee #{appexe}"))
end

from	= tmplocation(appexe)
to	= oldlocation(appexe)

File.copy(from, to)	unless from == to
